var version  = "0.3.1";               // 版本
var flagSimp = "simplified";          // 簡體
var flagTrad = "traditional";         // 繁體
var t2s = require('./dict/t2s.json'); // 繁轉簡 對照表
var s2t = require('./dict/s2t.json'); // 簡轉繁 對照表
var maxSTLen = 1;                     // 簡轉繁 最長的詞句
var maxTSLen = 1;                     // 繁轉簡 最長的詞句

var TongWen = (function () {
    var that = this;

    return {
        version       : version,
        flagSimp      : flagSimp,
        flagTrad      : flagTrad,
        addS2TTable   : addS2TTable,
        addT2STable   : addT2STable,
        convert       : convert,
        s2t           : function (str) { return convert(str, flagTrad); },
        t2s           : function (str) { return convert(str, flagSimp); }
    }
})();

// 新增 簡轉繁 對照表
function addS2TTable(table) {
    for (var i in table) {
        maxSTLen = Math.max(maxSTLen, table[i].length);
        s2t[i] = table[i];
    }
}

// 新增 繁轉簡 對照表
function addT2STable(table) {
    for (var i in table) {
        maxTSLen = Math.max(maxTSLen, table[i].length);
        t2s[i] = table[i];
    }
}

function setZhFlag(doc, zhflag) {
    doc.documentElement.setAttribute("zhtongwen", zhflag);
};

function getZhFlag(doc) {
    var zhflag = "";
    if (doc && doc.documentElement) {
        zhflag = doc.documentElement.getAttribute("zhtongwen");
        if (zhflag == null) zhflag = "";
    }
    return zhflag;
};

// 繁簡轉換
function convert(str, zhflag) {
    var leng = 4;
    var zmap = null;

    if (zhflag == flagSimp) {
        // 繁轉簡
        zmap = t2s;
        leng = Math.min(maxTSLen, str.length);
    } else {
        // 簡轉繁
        zmap = s2t;
        leng = Math.min(maxSTLen, str.length);
    }

    // 單字轉換
    str = str.split("");
    for (var i = 0, c = str.length; i < c; i++) {
        str[i] = zmap[str[i]] || str[i];
    }
    str = str.join("");

    // 詞彙轉換
    var txt = "", s = "", bol = true;
    for (var i = 0, c = str.length; i < c;) {
        bol = true;
        for (var j = leng; j > 1; j--) {
            s = str.substr(i, j);
            if (s in zmap) {
                txt += zmap[s];
                i += j;
                bol = false;
                break;
            }
        }

        if (bol) {
            txt += str.substr(i, 1);
            i++;
        }
    }
    if (txt != "") str = txt;
    return str;
}

module.exports = TongWen;
